#==============================================================================
#
#  $Id: senders.py,v 1.2 2000/04/17 12:36:02 mike Exp $
#
"""
   Module to house classes for sending e-mail messages.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: senders.py,v $
#  Revision 1.2  2000/04/17 12:36:02  mike
#  fixes to SMTP code
#
#
#==============================================================================

class SendError(Exception):
   """
      Raised when a @Sender is unable to send a message.
   """

class Sender:
   def sendMessage(self, msg):
      """
         parms:
            msg::
               [@spug.mail.message.Message] message to be sent.
      """
      raise NotImplementedError()

import smtplib
try:
   from cStringIO import StringIO
except ImportError:
   from StringIO import StringIO

class SMTPSender(Sender):
   """
      Sends messages via SMTP.
   """
   
   def __init__(self, smtpHost = 'localhost'):
      self.host = smtpHost
      self.smtp = smtplib.SMTP(smtpHost)
   
   def _getEmailAddrs(self, headers, headerName):
      # return a list of e-mail addresses from the message header
      if headers.has_key(headerName):
         return map(lambda x: x[1], headers.asAddressList(headerName).addresslist)
      else:
         return []
   
   def sendMessage(self, msg):
      "Implements @Sender.sendMessage()"
      buf = StringIO()
      msg.writeTo(buf, 1)
      
      # get the e-mail addresses of the sender and recipients
      sender = msg.headers.asAddressList('from').addresslist[0][1]
      recips = self._getEmailAddrs(msg.headers, 'to') + \
         self._getEmailAddrs(msg.headers, 'cc') + \
         self._getEmailAddrs(msg.headers, 'bcc')
      self.smtp.sendmail(sender, recips, buf.getvalue())

import popen2, string

class ShellCmd(Sender):

   """
      *ShellCmd* sends a message by simply piping it to a shell command
      with popen2.
      
      Any output from the shell command will be interpreted as an error
      and displayed in the message text.
      
      Each message will be piped to a unique invocation of the command.
   """

   def __init__(self, cmd):
      self.cmd = cmd

   def sendMessage(self, msg):   
      "Implements @Sender.sendMessage()"
      src, dst = popen2.popen2(self.cmd)
      msg.writeTo(dst, 0)
      dst.close()
      err = string.strip(src.read())
      if err:
         raise SendError(err)
